;*****************************************************************************
;				NICST TOOL
;
;  Uses a downhill simplex (ifit=0) and polynominal (ifit=1) fits to a quadratic 
;  equation, as well as a polynominal fit to a cubic equation (ifit=2), 
;  relating dn to radiance. 
;
;  Input:
;  1) Input file.
;
;  Output:
;  1) ncollects
;  2) gc[collect,band,HAM,detector,subsample,ifit] -- gain corrcetion per fit
;  3) rg[band,detector,a0 a1 a2 a3,gain correction,ifit] -- radiometric gain (ham and subsample averaged) per fit.
;  4) rg_1[band,HAM,detector,subsample,a0 a1 a2 a3,gain correction,ifit] -- radiometric gain per fit.
;  5) dL_fit[collect,band,HAM,detector,subsample,gain correction,ifit] -- fitting residual per fit.
;  6) rrcu[band,HAM,detector,subsample,gain correction,ifit] -- radiometric response characterization uncertainty per fit.
;  7) rrnl[band,HAM,detector,subsample,gain correction,ifit] -- radiometric response non-linearity per fit.
;  
;  v1.0 - 03/27/2008 - Jeff McIntire
;*****************************************************************************

;*****************************************************************************
;  Function for the simplex.
;*****************************************************************************

FUNCTION FUNC, Y

COMMON X1,b
COMMON X2,id
COMMON X3,ih
COMMON X4,is
COMMON X5,ncollects
COMMON X6,dn
COMMON X7,rad_BCS
COMMON X8,gc
COMMON X9,lcol
COMMON X10,hcol

XX=fltarr(ncollects)

ZZ=0
FOR col=lcol, hcol-1 DO BEGIN
	XX[col]=Y[0]+Y[1]*dn[col,b,ih,id,is,0]+Y[2]*dn[col,b,ih,id,is,0]^2-gc[col,b,ih,id,is]*rad_BCS[col,b,ih,id,is]

	ZZ=ZZ+XX[col]^2
ENDFOR

RETURN,ZZ

END

;*****************************************************************************
;  Call the simplex to get the radiometric gains.
;*****************************************************************************

PRO RC5_simplex, infile

;*****************************************************************************
;  Read in input file, sav TEB file, and make some definitions.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
Free_Lun,lun

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

infname1=outpath+a2[a3-2]+'.'+a2[a3-1]+'.dn.sav'
infname2=outpath+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
restore,infname1
restore,infname2

restore,'/home/jmcintire/VIIRS_Data/general.TEB.sav'

outfile=outpath+a2[a3-2]+'.'+a2[a3-1]+'.rg.sav'

COMMON X1,b
COMMON X2,id
COMMON X3,ih
COMMON X4,is
COMMON X5,ncollects
COMMON X6,dn
COMMON X7,rad_BCS
COMMON X8,gc
COMMON X9,lcol
COMMON X10,hcol

x=dblarr(ncollects)
z=dblarr(ncollects)

gc=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,3)
rg_1=dblarr(nbands_TEB,nham,maxdet,maxsubsets,4,2,3)
rg=dblarr(nbands_TEB,maxdet,4,2,3)
dL_fit=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,2,3)
mean_dL_fit=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2,3)
abs_max_dL_fit=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2,3)
stddev_dL_fit=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2,3)
rrcu=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2,3)
rrnl=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2,3)

;*****************************************************************************
;  Fitting using the downhill simplex of AMEOBA and POLY_FIT (Quadratic and Cubic).
;*****************************************************************************

;  0 for AMEOBA and 1 for POLY_FIT.
FOR ifit=0, 1 DO BEGIN
	;  0 for no gain correction and 1 for gain correction.
	FOR cor=0, 1 DO BEGIN
		IF cor EQ 0 THEN gc[*,*,*,*,*,ifit]=1
		IF cor EQ 1 THEN BEGIN
			FOR b=0, nbands_TEB-1 DO BEGIN
				FOR col=0, ncollects-1 DO BEGIN
					FOR is=0, subsets_TEB[b]-1 DO BEGIN
						FOR ih=0, nham-1 DO BEGIN
								gc[col,b,ih,*,is,ifit]=(rg[b,*,0,0,ifit] $
									+rg[b,*,1,0,ifit]*dn_obc[col,b,ih,*,is,0] $
									+rg[b,*,2,0,ifit]*dn_obc[col,b,ih,*,is,0]^2) $
									/rad_obc[col,b,ih,*,is]
						ENDFOR
					ENDFOR
				ENDFOR
			ENDFOR
		ENDIF
		IF ifit EQ 1 THEN gc[*,*,*,*,*,ifit+1]=gc[*,*,*,*,*,ifit]
		FOR b=0, nbands_TEB-1 DO BEGIN
			FOR is=0, subsets_TEB[b]-1 DO BEGIN
				FOR ih=0, nham-1 DO BEGIN
					FOR id=0, detectors_TEB[b]-1 DO BEGIN
						lcol=0
						hcol=ncollects
						FOR col=0, ncollects-1 DO BEGIN
							IF rad_BCS[col,b,ih,id,is] LT L_min_TEB[b] THEN lcol=lcol+1
							IF rad_BCS[col,b,ih,id,is] GT L_max_TEB[b] THEN hcol=hcol-1
						ENDFOR

						IF ifit EQ 0 THEN BEGIN
							Y=[0,0,0]
							P=[-.5,.004,.0]
							S=[.5,.004,.001]

							A=AMOEBA(0.001,FUNCTION_VALUE=Y,P0=P,SCALE=S)
							rg_1[b,ih,id,is,0:2,cor,ifit]=A[0:2]

							IF Y[0] GT 0.01 THEN print,b,id,ih,is,1/A[1],Y[0]
							IF Y[0] GT 0.01 THEN BEGIN
								FOR m=0, 5 DO BEGIN
									Y=[0,0,0]
									P=[-.45+m*.05,.004,.0]
									S=[.45-m*.05,.004,.001]

									A=AMOEBA(0.001,FUNCTION_VALUE=Y,P0=P,SCALE=S)
									rg_1[b,ih,id,is,0:2,cor,ifit]=A[0:2]

									print,m,b,id,ih,is,1/A[1],Y[0]
									IF Y[0] LT 0.01 THEN goto,skip
								ENDFOR
								skip:
							ENDIF
						ENDIF ELSE BEGIN
							x=dn[lcol:hcol-1,b,ih,id,is,0]
							z=gc[lcol:hcol-1,b,ih,id,is,ifit]*rad_BCS[lcol:hcol-1,b,ih,id,is]
							result1=POLY_FIT(x,z,2)
							result2=POLY_FIT(x,z,3)

							rg_1[b,ih,id,is,0:2,cor,ifit]=result1[0:2]
							rg_1[b,ih,id,is,*,cor,ifit+1]=result2[*]
						ENDELSE

;*****************************************************************************
;  Get the fitting residual, fractional fitting residual, radiometric
;  response characterization uncertainty, and radiometric response non-linearity.
;*****************************************************************************

						FOR col=0, ncollects-1 DO BEGIN
							IF ifit EQ 0 THEN BEGIN
								dL_fit[col,b,ih,id,is,cor,ifit]= $
									(gc[col,b,ih,id,is]*rad_BCS[col,b,ih,id,is] $
									-A[0]-A[1]*dn[col,b,ih,id,is] $
									-A[2]*dn[col,b,ih,id,is]^2) $
									/(gc[col,b,ih,id,is]*rad_BCS[col,b,ih,id,is])
							ENDIF ELSE BEGIN
								dL_fit[col,b,ih,id,is,cor,ifit]= $
									(gc[col,b,ih,id,is]*rad_BCS[col,b,ih,id,is] $
									-result1[0]-result1[1]*dn[col,b,ih,id,is] $
									-result1[2]*dn[col,b,ih,id,is]^2) $
									/(gc[col,b,ih,id,is]*rad_BCS[col,b,ih,id,is])
								dL_fit[col,b,ih,id,is,cor,ifit+1]= $
									(gc[col,b,ih,id,is]*rad_BCS[col,b,ih,id,is] $
									-result2[0]-result2[1]*dn[col,b,ih,id,is] $
									-result2[2]*dn[col,b,ih,id,is]^2) $
									/(gc[col,b,ih,id,is]*rad_BCS[col,b,ih,id,is])
							ENDELSE
						ENDFOR
						mean_dL_fit[b,ih,id,is,cor,ifit]= $
							sig_mean(dL_fit[lcol:hcol-1,b,ih,id,is,cor,ifit],3,1)
						IF ifit NE 0 THEN mean_dL_fit[b,ih,id,is,cor,ifit+1]= $
							sig_mean(dL_fit[lcol:hcol-1,b,ih,id,is,cor,ifit+1],3,1)
						abs_max_dL_fit[b,ih,id,is,cor,ifit]= $
							max(abs(dL_fit[lcol:hcol-1,b,ih,id,is,cor,ifit]))
						IF ifit NE 0 THEN abs_max_dL_fit[b,ih,id,is,cor,ifit+1]= $
							max(abs(dL_fit[lcol:hcol-1,b,ih,id,is,cor,ifit+1]))
						stddev_dL_fit[b,ih,id,is,cor,ifit]= $
							sig_mean(dL_fit[lcol:hcol-1,b,ih,id,is,cor,ifit],3,2)
						IF ifit NE 0 THEN stddev_dL_fit[b,ih,id,is,cor,ifit+1]= $
							sig_mean(dL_fit[lcol:hcol-1,b,ih,id,is,cor,ifit+1],3,2)

						rrcu[b,ih,id,is,cor,ifit]=sqrt(mean_dL_fit[b,ih,id,is,cor,ifit]^2 $
							+stddev_dL_fit[b,ih,id,is,cor,ifit]^2)/0.001
						IF ifit NE 0 THEN rrcu[b,ih,id,is,cor,ifit+1] $
							=sqrt(mean_dL_fit[b,ih,id,is,cor,ifit]^2 $
							+stddev_dL_fit[b,ih,id,is,cor,ifit]^2)/0.001
						rrnl[b,ih,id,is,cor,ifit]=abs_max_dL_fit[b,ih,id,is,cor,ifit] $
							/(0.01*L_max_TEB[b,0])
						IF ifit NE 0 THEN rrnl[b,ih,id,is,cor,ifit+1] $
							=abs_max_dL_fit[b,ih,id,is,cor,ifit]/(0.01*L_max_TEB[b,0])
if b eq 0 then print,ifit,cor,b,is,ih,id,mean_dL_fit[b,ih,id,is,cor,ifit]^2,stddev_dL_fit[b,ih,id,is,cor,ifit]^2
					ENDFOR
				ENDFOR
			ENDFOR
			IF band_type_TEB[b] EQ 1 THEN rg[b,*,*,cor,ifit]=(rg_1[b,0,*,0,*,cor,ifit]+rg_1[b,0,*,1,*,cor,ifit] $
				+rg_1[b,1,*,0,*,cor,ifit]+rg_1[b,1,*,1,*,cor,ifit])/4
			IF ifit NE 0 AND band_type_TEB[b] EQ 1 THEN rg[b,*,*,cor,ifit+1] $
				=(rg_1[b,0,*,0,*,cor,ifit+1]+rg_1[b,0,*,1,*,cor,ifit+1] $
				+rg_1[b,1,*,0,*,cor,ifit+1]+rg_1[b,1,*,1,*,cor,ifit+1])/4
			IF band_type_TEB[b] EQ 0 THEN rg[b,*,*,cor,ifit]=(rg_1[b,0,*,0,*,cor,ifit]+rg_1[b,1,*,0,*,cor,ifit])/2
			IF ifit NE 0 AND band_type_TEB[b] EQ 0 THEN rg[b,*,*,cor,ifit+1] $
				=(rg_1[b,0,*,0,*,cor,ifit+1]+rg_1[b,1,*,0,*,cor,ifit+1])/2

		;	print,b,mean(1/rg[b,0:detectors_TEB[b]-1,1,cor,ifit])
		ENDFOR
	ENDFOR
ENDFOR

save,/compress,ncollects,gc,rg,rg_1,dL_fit,rrcu,rrnl,filename=outfile

END
