;*****************************************************************************
;				NICST TOOL
;
;  EDU RC-5 test for TEB bands. Calculates the Blackbody Calibration Source (BCS)
;  radiance.
;
;  Input:
;  1) RVS data.
;  2) Band dependent reflectence of the RTA (p).
;  3) Temperatures of instrument (HAM, RTA, CAV, OBC BB, SH, CPFA, OMM,
;     LWASP, MWASP, BCS, and SVS).
;  4) RSR data.
;
;  Output:
;  1) rad_BCS[collect,band,HAM,detector,subsample]
;  2) rad_OBC[collect,band,HAM,detector,subsample]
;  3) rad_p1[band,collect,2] -- 0 for BCS and 1 for SVS.
;  4) rad_p2[band,HAM,collect,5] -- 0 for HAM, 1 for RTA, 2 for CAV, 3 for SH, 
;     and 4 for OBC.
;  5) rvs[band,subsample,detector,HAM,2] -- 0 for BCS and 1 for SVS.
;  6) ncollects
;  
;  v1.0 - 03/27/2008 - Jeff McIntire
;*****************************************************************************

;*****************************************************************************
;  Main program.
;*****************************************************************************

PRO RC5_rad, infile

;*****************************************************************************
;  Read in input file and sav TEB file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)		; RVS .sav file.
param_in=strarr(1)		; EDU coeff .sav file.
rsrpath=strarr(1)		; RSR .sav file.

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
Free_Lun,lun

n_collect,inpath,ncollects

restore,'/home/jmcintire/VIIRS_Data/general.TEB.sav'

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

restore,param_in

max_scan=100					; # of scans.

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

outfile=''
FOR i=0, a3-1 DO BEGIN
	outfile=outfile+'/'+a2[i]
ENDFOR
outfile=outfile+'/'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
print,outfile

rad_p1=dblarr(nbands_TEB,ncollects,2)
rad_p2=dblarr(nbands_TEB,nham,ncollects,5)

rvs=dblarr(nbands_TEB,maxsubsets,maxdet,nham,3)

rad_BCS=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets)
rad_OBC_1=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets)
rad_OBC=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets)

;*****************************************************************************
;  Calculate angles.
;*****************************************************************************

win_num=4

; Starting position of the collect window in samples.
MSample_offset=1064*win_num
ISample_offset=2128*win_num

Msample_max=1850+MSample_offset
Msample_min=1650+MSample_offset
Isample_max=3700+ISample_offset
Isample_min=3300+ISample_offset

; Center sample numbers.
Msample_center=0.5*(Msample_max+Msample_min)
Isample_center=0.5*(Isample_max+Isample_min)

; Bore-sight scan angles.
Theta_M=((Msample_center-33.5)*0.017785-56.063)*(!pi/180)
Theta_I=(0.5*(Isample_center-67)*0.017785-56.063)*(!pi/180)

Theta_SVS=-65.68*(!pi/180)
;Theta_SVS=-65*(!pi/180)
Theta_OBC=157.43*(!pi/180)

Theta_ref=46*(!pi/180)

; HAM angle of incidence.
Phi_perpendicular=28.6*(!pi/180)
Phi=dblarr(nbands_TEB,3)				; All bands for BCS (or TMC for M13), SVS, and OBC.

FOR b=0, nbands_TEB-1 DO BEGIN
	IF b LE 1 THEN Phi[b,0]=acos(cos(Phi_perpendicular)*cos(0.5*(Theta_I-Theta_ref)))*(180/!pi)
	IF b GT 1 THEN Phi[b,0]=acos(cos(Phi_perpendicular)*cos(0.5*(Theta_M-Theta_ref)))*(180/!pi)
	Phi[b,1]=acos(cos(Phi_perpendicular)*cos(0.5*(Theta_SVS-Theta_ref)))*(180/!pi)
	Phi[b,2]=acos(cos(Phi_perpendicular)*cos(0.5*(Theta_OBC-Theta_ref)))*(180/!pi)
ENDFOR

;*****************************************************************************
;  Read in scan averaged temperatures for the HAM, RTA, CAV, OBC BB,
;  SH, CPFA, OMM, LWASP, MWASP, BCS, and SVS.
;*****************************************************************************

restore,outpath+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
print,outpath+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'

;*****************************************************************************
;  Calculate the radiances from Plank's radiation law for the BCS, SVS, 
;  HAM, RTA, CAV, SH, and OBC.
;*****************************************************************************

; Loop over BCS, rad_p1(i=0), SVS, rad_p1(i=1), HAM, rad_p2(i=0), RTA, rad_p2(i=1),
; CAV, rad_p2(i=2), SH, rad_p2(i=3), OBC, rad_p2(i=4).
FOR b=0, nbands_TEB-1 DO BEGIN
	FOR col=0, ncollects-1 DO BEGIN
		rad_p1[b,col,0]=Planck(T_bcs[col],lambda_TEB[b])
		rad_p1[b,col,1]=Planck(T_svs[col],lambda_TEB[b])
		FOR ih=0, nham-1 DO BEGIN
			rad_p2[b,ih,col,0]=Planck(T_ham[ih,col],lambda_TEB[b])
			rad_p2[b,ih,col,1]=Planck(T_rta[ih,col],lambda_TEB[b])
			rad_p2[b,ih,col,2]=Planck(T_cav[ih,col],lambda_TEB[b])
			rad_p2[b,ih,col,3]=Planck(T_sh[ih,col],lambda_TEB[b])
			rad_p2[b,ih,col,4]=Planck(T_obc[ih,col],lambda_TEB[b])
		ENDFOR
	ENDFOR
ENDFOR

;*****************************************************************************
;  Integrate the RSR and L over wavelength.
;*****************************************************************************

;restore,rsrpath

;FOR b=0, nbands_TEB-1 DO BEGIN
;	get_avg_l,rsrf,lampf,tranf,b,detectors_TEB[b],0
;ENDFOR

;*****************************************************************************
;  Calculate the RVS.
;*****************************************************************************

; Restore RVS data called COEFF=dblarr(# bands,subsample,# detectors,HAM side,three fit parameters (ao,a1,a2)).
restore,rvs_in
print,rvs_in

FOR b=0, nbands_TEB-1 DO BEGIN

	; 0 for BCS, 1 for SVS, and 2 for OBC.
	FOR j=0, 2 DO BEGIN
		IF band_type_TEB[b] EQ 1 THEN BEGIN
			rvs[b,*,*,*,j]=(COEF[b+6,*,*,*,0]+COEF[b+6,*,*,*,1]*Phi[b,j]+COEF[b+6,*,*,*,2]*Phi[b,j]^2) $
				/(COEF[b+6,*,*,*,0]+COEF[b+6,*,*,*,1]*38.53+COEF[b+6,*,*,*,2]*38.53^2)
		ENDIF
		IF band_type_TEB[b] EQ 0 THEN BEGIN
			rvs[b,*,*,*,j]=(COEF[b-2,*,*,*,0]+COEF[b-2,*,*,*,1]*Phi[b,j]+COEF[b-2,*,*,*,2]*Phi[b,j]^2) $
				/(COEF[b-2,*,*,*,0]+COEF[b-2,*,*,*,1]*38.53+COEF[b-2,*,*,*,2]*38.53^2)
		ENDIF
	ENDFOR
ENDFOR
print,Phi[*,0]
print,Phi[*,1]

;*****************************************************************************
;  Calculate the BCS and OBC radiances.
;*****************************************************************************

FOR b=0, nbands_TEB-1 DO BEGIN
	FOR ih=0, nham-1 DO BEGIN
		FOR col=0, ncollects-1 DO BEGIN
			FOR is=0, subsets_TEB[b]-1 DO BEGIN
				rad_BCS[col,b,ih,*,is]=p[b]*(rvs[b,is,*,ih,0]*rad_p1[b,col,0]-rvs[b,is,*,ih,1]*rad_p1[b,col,1]) $
					+(rvs[b,is,*,ih,1]-rvs[b,is,*,ih,0])*(rad_p2[b,ih,col,0]-(1-p[b])*rad_p2[b,ih,col,1])

				rad_OBC_1[col,b,ih,*,is]=emiss_bb*rad_p2[b,ih,col,4]+(1-emiss_bb)*(f_cav*rad_p2[b,ih,col,2] $
					+f_sh*rad_p2[b,ih,col,3]+f_rta*rad_p2[b,ih,col,1])

				rad_OBC[col,b,ih,*,is]=p[b]*(rad_OBC_1[col,b,ih,*,is]-rvs[b,is,*,ih,1]*rad_p1[b,col,1]) $
					+(rvs[b,is,*,ih,1]-1)*(rad_p2[b,ih,col,0]-(1-p[b])*rad_p2[b,ih,col,1])
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

save,/compress,ncollects,rad_BCS,rad_OBC,rad_p1,rad_p2,rvs,p,filename=outfile

END

