;*****************************************************************************
;  Calculate band and detector averaged sis radiance.
;
;  Inputs:
;  1) rsrf -- RSR data.
;  2) lampf -- lamp levels.
;  3) tranf -- transmittence.
;  4) band -- band index.
;  5) flag -- select spine (0) or interpolation (1).
;
;  Outputs:
;  1) l_avg -- Result of integral.
;
;  v1 - Chunhui Pan
;  v2 - 4/15/2008 - Jeff McIntire
;*****************************************************************************

FUNCTION get_avg_l, rsrf,lampf,tranf,band,numdet,flag

; Number of resample points/resolution.
npts=512     

; Read rsr as function of wavelength based on band and detector.
arr=ddread(rsrf)
rsr=reform(arr[3,*])

ix=WHERE(rsr GE 0,count) 
IF count GT 0 THEN BEGIN
	arr=arr[*,ix]
	rsr_band=fix(reform(arr[0,*]))
	rsr_det=fix(reform(arr[1,*]))
	rsr_wl=double(reform(arr[2,*]))/1000. 	; unit: microns
	rsr=double(reform(arr[3,*]))
ENDIF ELSE print,'Check rsr file =>',rsrf

; Sort rsr (finds the channels present and places them in ascending order). 
det=rsr_det(sort(rsr_det))
det=det(uniq(det))

; Read transmittance as a function of wave length.
arr=ddread(tranf)
t_wl=reform(arr[0,*])
t=reform(arr[1,*])

; Check number of detectors for a given band.
ndet=n_elements(det)
IF ndet NE numdet THEN BEGIN
    print,'The band number does not match with the rsr file.'
    stop;
ENDIF

l_avg=fltarr(ndet,n_elements(lampf))

; Loop for all lamp levels.
FOR ifs=0, n_elements(lampf)-1 DO BEGIN

; Read lamp radiances. 
	arr=ddread(lampf[ifs])
	lamp_wl=reform(arr[0,*])
	l=reform(arr[1,*])

; Do integration.
	FOR idet=0, ndet-1 DO BEGIN
    		ix=WHERE(rsr_det EQ det[idet])
		rsr_w=rsr_wl[ix]
    		rsr_org=rsr[ix]

; Refine wave length for resampling.
    		minwl=min(rsr_w,max=maxwl)
    		wave_fine=minwl+(maxwl-minwl)*(findgen(npts)/(npts-1))

; Check rsr for wave length duplicates. 
    		ix=sort(rsr_w)   
    		rsr_w=rsr_w[ix]
    		rsr_org=rsr_org[ix]

; Check rsr uniqueness for interpolation.
    		ix=uniq(rsr_w)
    		rsr_w=rsr_w[ix]
    		rsr_org=rsr_org[ix]
    		rsr_fine=spline(rsr_w,rsr_org,wave_fine)

; Check for convergence of the interpolation.
    		ix=WHERE(finite(rsr_fine) NE 1,count)
    		IF count GT 0 THEN print,'The splined data did not converge.'

; Resample by interpolation or spline.
		IF flag EQ 0 THEN BEGIN
    			t_fine=spline(t_wl,t,wave_fine)
    			l_fine=spline(lamp_wl,l,wave_fine)
		ENDIF
		IF flag EQ 1 THEN BEGIN
	  		t_fine=interpol(t_wl,t,wave_fine)
	  		l_fine=interpol(lamp_wl,l,wave_fine)
		ENDIF
    
; Do integration.
    		IF flag EQ 0 THEN l_avg[idet,ifs]=total(rsr_fine*l_fine)/total(rsr_fine)
    		IF flag EQ 1 THEN l_avg[idet,ifs]=total(rsr_fine*t_fine*l_fine)/total(rsr_fine)
  	ENDFOR                        
ENDFOR 
                       
RETURN,l_avg

END
